program pdfsearch;

{$mode objfpc}

uses
  browserapp, JS, Classes, SysUtils, Web, pdfjs, searchpdf, types , strutils;

Const
  ScaleStep = 0.2;

var
  pdfjsLib : TPDFJSStatic; external name 'pdfjsLib';

Type
  TCurrentPageInfo = record
    page : TPDFPageProxy;
    viewport : TPDFPageViewport;
    renderContext: TPDFRenderParams;
    renderTask : TPDFRenderTask;
    viewportParams : TViewportParameters;
  end;

  { TMyApplication }

  TMyApplication = class(TBrowserApplication)
    divpdfViewer,
    FTextlayer,
    lblFileLocation,
    lblZoom : TJSHTMLElement;
    FCurrentPageInfo : TCurrentPageInfo;

    btnLoad,
    btnUpload,
    btnZoomIn,
    btnZoomOut,
    btnPrevious,
    btnFirst,
    btnNext,
    btnLast,
    btnSearch,
    btnClosePane,
    btnCloseSearchResult : TJSHTMLButtonElement;
    edtSearchInResults,
    edtSearch,
    edtPDFFile,
    edtPageNo : TJSHTMLInputElement;
    pdfDoc : TPDFDocumentProxy;
    FPageNum : Integer; // 1
    FpageRendering : Boolean; // False
    FpageNumPending : Integer;// -1
    Fscale : Double ; // 0.8,
    FCanvas : TJSHTMLCanvasElement;
    FCtx : TJSCanvasRenderingContext2D; // canvas.getContext('2d');
    pnlSidebar,
    pnlSearch,
    pnlResults,
    lblCurrentSearchResult,
    lblSearchResultCount,
    FSearchResultsPopover,
    FSearchNextResult,
    FSearchPreviousResult : TJSHTMLElement;
    FSearchResultPages : Array of Integer;
    FCurrentSearchResultPage : Integer;
    FSearch: TPDFSearch;
    procedure doRun; override;
    procedure BindElements;
  private
    // Load
    function DoLoadFile(Event: TEventListenerEvent): boolean;
    procedure HighLight(El: TJSHTMLElement; aRegex: TJSRegexp);
    procedure onLoad(aEvent: TJSEvent);
    function RenderText(aValue: JSValue): JSValue;
    procedure ShowPDF(aSource: TPDFSource);
    procedure StartPDFRender;
    procedure queueRenderPage(aNum: Integer);
    procedure renderPage(aNum: Integer);
    Procedure DisplayFileLocation(const aLocation : String);
    // Navigation
    function DoPageKeyUp(aEvent: TJSKeyBoardEvent): boolean;
    procedure onFirstPage(aEvent: TJSEvent);
    procedure onLastPage(aEvent: TJSEvent);
    procedure onNextPage(aEvent: TJSEvent);
    procedure onPrevPage(aEvent: TJSEvent);
    // Zoom
    procedure DisplayZoom;
    procedure onZoomIn(aEvent: TJSEvent);
    procedure onZoomOut(aEvent: TJSEvent);
    // Search
    procedure DoSearch;
    procedure onNextSearchResult(aEvent: TJSEvent);
    procedure onPreviousSearchResult(aEvent: TJSEvent);
    function DoSearchKeyUp(aEvent: TJSKeyBoardEvent): boolean;
    Procedure DoSearchFocus(aEvent : TJSEvent);
    function DoSearchResultsKeyUp(aEvent: TJSKeyBoardEvent): boolean;
    Function ShowResultMatch(Idx : Integer; aResult : TPDFSearchMatch) : TJSHTMLElement;
    procedure ShowResultPane;
    procedure DoShowMatch(aEvent: TJSEvent);
    procedure HideResultPane;
    procedure ClearResultPane;
    procedure ShowSearchResultMatchCount;
    procedure ShowSearchResultPage(aIdx: Integer);
    procedure onClosePane(aEvent: TJSEvent);
    procedure onSearch(aEvent: TJSEvent);
    procedure onCloseSearch(aEvent: TJSEvent);
    function TextDone(aValue: JSValue): JSValue;
  end;


Procedure TMyApplication.HighLight(El : TJSHTMLElement; aRegex : TJSRegexp);

Var
  S,aText,aLeft : String;
  Matches : TStringDynArray;
  aLast : Integer;
  aSpan : TJSHTMLElement;

begin
  aText:=El.innerText;
  Matches:=aRegex.exec(aText);
  aLast:=1;
  While Assigned(Matches) do
    begin
    Writeln('Found match in ',aText,' : ',Matches[0]);
    if aLast=1 then
      EL.InnerHTML:='';
    S:=Copy(aText,aLast,aRegex.lastIndex-Length(Matches[0]));
    El.AppendChild(document.createTextNode(S));
    aSpan:=TJSHTMLElement(document.createElement('span'));
    aSpan.InnerText:=Matches[0];
    aSpan.className:='highlight';
    El.AppendChild(aSpan);
    aLast:=aRegex.LastIndex+1;
    Matches:=aRegex.exec(El.innerText);
    end;
  if aLast<length(aText) then
    begin
    S:=Copy(aText,aLast,Length(aText)-aRegex.lastIndex);
    El.AppendChild(document.createTextNode(S));
    end;
end;

Function TMyApplication.TextDone(aValue: JSValue) : JSValue;


var
  El : TJSELement;
  aRegex : TJSRegExp;
  aReg,aTerm : String;

begin
  aTerm:=edtSearch.Value;
  if aTerm='' then exit;
  aReg:=Format('%s',[aTerm]);
  aRegEx:=TJSRegExp.New(aReg,'gi');
  El:=FTextlayer.firstElementChild;
  While Assigned(El) do
    begin
    if (el is TJSHTMLElement) and SameText(El.tagName,'span') then
      HighLight(TJSHTMLElement(El),aRegex);
    El:=El.nextElementSibling;
    end;
end;

Function TMyApplication.RenderText(aValue: JSValue) : JSValue;

Var
  aContent : TTextContent absolute aValue;
  aTextRender : TPDFJSRenderTextLayerParameters;

begin
  FTextlayer.InnerHTML:='';
  aTextRender:=TPDFJSRenderTextLayerParameters.New;
  aTextRender.container:=FTextlayer;
  aTextRender.isOffscreenCanvasSupported:=true;
  aTextRender.textContentSourceItems:=aContent;
  aTextRender.viewPort:=FCurrentPageInfo.viewport;
  pdfjsLib.renderTextLayer(aTextRender).promise.&then(@TextDone);
end;

procedure TMyApplication.renderPage(aNum: Integer);

  function renderOK(aValue : JSValue) : JSValue;

  Var
    N : Integer;

  begin
    FPageRendering:=false;
    if (FPageNumPending <> -1) then
      begin
      N:=FPageNumPending;
      FpageNumPending:=-1;
      renderPage(N);
      end;
    Result:=True;
    FCurrentPageInfo.page.getTextContent().&Then(@RenderText);
    divpdfViewer.style.setProperty('--scale-factor',FloatToStr(FScale))
  end;

  function havePage (aValue : JSValue) : JSValue;

  var
    page : TPDFPageProxy absolute aValue;

  begin
    FCurrentPageInfo.Page:=page;
    FCurrentPageInfo.viewportParams:=TViewportParameters.new;
    FCurrentPageInfo.viewportParams.scale:=FScale;
    FCurrentPageInfo.viewport:=page.getViewport(FCurrentPageInfo.viewportParams);
    Fcanvas.height := FCurrentPageInfo.viewport.height;
    Fcanvas.width := FCurrentPageInfo.viewport.width;
    FCurrentPageInfo.renderContext:=TPDFRenderParams.New;
    FCurrentPageInfo.renderContext.canvasContext:=Fctx;
    FCurrentPageInfo.renderContext.viewport:=FCurrentPageInfo.viewport;
    FCurrentPageInfo.renderTask:=page.render(FCurrentPageInfo.renderContext);
    FCurrentPageInfo.renderTask.promise.&then(@renderOK);
    Result:=True;
  end;

begin
  FpageRendering:=True;
  pdfDoc.getPage(aNum).&then(@HavePage);
  edtPageNo.Value:=IntToStr(anum);
end;

procedure TMyApplication.queueRenderPage(aNum: Integer);

begin
  if FPageRendering then
    FpageNumPending:=aNum
  else
    renderPage(aNum)
end;

procedure TMyApplication.onPrevPage(aEvent : TJSEvent);

begin
  if (FpageNum <= 1) then exit;
  Dec(FpageNum);
  queueRenderPage(FpageNum);
end;


procedure TMyApplication.onFirstPage(aEvent : TJSEvent);

begin
  if not assigned(pdfDoc) then exit;
  FPageNum:=1;
  queueRenderPage(FpageNum);
end;

procedure TMyApplication.onLastPage(aEvent : TJSEvent);

begin
  if not assigned(pdfDoc) then exit;
  FPageNum:=pdfDoc.numPages;
  queueRenderPage(FpageNum);
end;

procedure TMyApplication.onNextPage(aEvent : TJSEvent);

begin
  if not assigned(pdfDoc) then exit;
  if (FpageNum >= pdfDoc.numPages) then exit;
  Inc(FpageNum);
  queueRenderPage(FpageNum);
end;

procedure TMyApplication.onZoomIn(aEvent : TJSEvent);

begin
  if not assigned(pdfDoc) then exit;
  FScale:=FScale+ScaleStep;
  DisplayZoom;
  queueRenderPage(FpageNum);
end;

procedure TMyApplication.onZoomOut(aEvent : TJSEvent);

begin
  if not assigned(pdfDoc) then exit;
  if FScale<=ScaleStep*2 then
    exit;
  FScale:=FScale-ScaleStep;
  DisplayZoom;
  queueRenderPage(FpageNum);
end;

procedure TMyApplication.DisplayZoom;
begin
  lblZoom.innerText:=IntToStr(Round(FScale*100))+'%';
end;

procedure TMyApplication.DisplayFileLocation(const aLocation: String);
begin
  if aLocation<>'' then
    begin
    document.title:='View PDF file: '+aLocation;
    lblFileLocation.InnerText:=': '+aLocation;
    end
  else
    begin
    document.title:='View PDF file';
    lblFileLocation.InnerText:='';
    end;
end;

function TMyApplication.DoLoadFile(Event: TEventListenerEvent): boolean;

Var
  aFileName : string;

  Function LoadFromFile(aData : JSValue) : JSValue;

  var
    aSource : TPDFSource;

  begin
    Result:=False;
    pdfDoc:=Nil;
    aSource:=TPDFSource.New;
    aSource.data:=aData;
    ShowPDF(aSource);
    DisplayFileLocation(aFileName);
  end;

begin
  Result:=False;
  if (edtPDFFile.files.length=1) then
    begin
    aFileName:=ExtractFileName(edtPDFFile.files[0].Name);
    edtPDFFile.files[0].arraybuffer._then(@loadFromFile);
    end;
end;

procedure MakeInactive(currentValue : TJSNode; currentIndex: NativeInt; list : TJSNodeList);

begin
  TJSHTMLElement(CurrentValue).classList.add('is-active');
end;

procedure TMyApplication.ShowPDF(aSource: TPDFSource);

  function pdfLoaded(aValue : JSValue) : JSValue;
  begin
    pdfDoc:=TPDFDocumentProxy(aValue);
    Writeln('PDF loaded');
    FpageNum:=1;
    QueueRenderPage(FpageNum);
    Result:=True;
  end;

  Procedure pdfLoadError(aReason : String);

  begin
    Writeln('PDF loading failed: '+aReason);
  end;

var
  loadingTask : TPDFLoadingTask;

begin
  loadingTask:=pdfjsLib.getDocument(aSource);
  loadingTask.promise.&then(@pdfLoaded,@pdfLoadError);
end;

procedure TMyApplication.StartPDFRender;

var
  aSource : TPDFSource;
  aURL : String;

begin
{  aUrl:=window.prompt('Please enter the URL of a PDF to view');
  if IsNull(aURL) then
    exit;}
  aUrl:='sample.pdf';
  aSource:=TPDFSource.New;
  aSource.URL:=aURL;
  ShowPdf(aSource);
  DisplayFileLocation(aURL);
end;

procedure TMyApplication.doRun;

begin
  FPageNum:=1;
  FPageRendering:=False;
  FPageNumPending:= -1;
  FScale:=1;
  // pdfjsLib.GlobalWorkerOptions.workerSrc:= 'https://mozilla.github.io/pdf.js/build/pdf.worker.js';
  pdfjsLib.GlobalWorkerOptions.workerSrc:= 'https://cdnjs.cloudflare.com/ajax/libs/pdf.js/4.0.269/pdf.worker.mjs';
  BindElements;
  StartPDFRender;
  DisplayZoom;
  Terminate;

end;

procedure TMyApplication.BindElements;
begin
  btnPrevious:=TJSHTMLButtonElement(GetHTMLElement('btnPrevious'));
  btnPrevious.addEventListener('click', @onPrevPage);
  btnFirst:=TJSHTMLButtonElement(GetHTMLElement('btnFirst'));
  btnFirst.addEventListener('click', @onFirstPage);
  btnNext:=TJSHTMLButtonElement(GetHTMLElement('btnNext'));
  btnNext.addEventListener('click', @onNextPage);
  btnLast:=TJSHTMLButtonElement(GetHTMLElement('btnLast'));
  btnLast.addEventListener('click', @onLastPage);
  btnLoad:=TJSHTMLButtonElement(GetHTMLElement('btnLoad'));
  btnLoad.addEventListener('click', @onLoad);
//  btnUpload:=TJSHTMLButtonElement(GetHTMLElement('btnUpload'));
//  btnUpload.addEventListener('click', @onUpLoad);

  btnZoomIn:=TJSHTMLButtonElement(GetHTMLElement('btnZoomIn'));
  btnZoomIn.addEventListener('click', @onZoomIn);
  btnZoomOut:=TJSHTMLButtonElement(GetHTMLElement('btnZoomOut'));
  btnZoomOut.addEventListener('click', @onZoomOut);
  lblZoom:=GetHTMLElement('lblZoom');
  lblFileLocation:=GetHTMLElement('lblFileLocation');
  edtPDFFile:=TJSHTMLInputElement(GetHTMLElement('edtPDFFile'));
  edtPDFFile.onchange:=@DoLoadFile;

  edtPageNo:=TJSHTMLInputElement(GetHTMLElement('edtPageNo'));
  edtPageNo.onkeyup:=@DoPageKeyUp;
  FCanvas:=TJSHTMLCanvasElement(GetHTMLElement('PDFCanvas'));
  FCtx:=TJSCanvasRenderingContext2D(FCanvas.getContext('2d'));

  FSearchResultsPopover:=GetHTMLElement('searchPopover');
  FSearchNextResult:=GetHTMLElement('btnNextSearchResult');
  FSearchNextResult.addEventListener('click', @OnNextSearchResult);
  FSearchPreviousResult:=GetHTMLElement('btnPreviousSearchResult');
  FSearchPreviousResult.addEventListener('click', @OnPreviousSearchResult);
  lblCurrentSearchResult:=GetHTMLElement('lblCurrentSearchResult');
  lblSearchResultCount:=GetHTMLElement('lblSearchResultCount');
  btnClosePane:=TJSHTMLButtonElement(GetHTMLElement('btnClosePane'));
  btnClosePane.addEventListener('click', @onClosePane);
  pnlSidebar:=TJSHTMLInputElement(GetHTMLElement('pnlSidebar'));
  pnlSearch:=TJSHTMLInputElement(GetHTMLElement('pnlSearch'));
  pnlResults:=TJSHTMLInputElement(GetHTMLElement('pnlResults'));
  btnSearch:=TJSHTMLButtonElement(GetHTMLElement('btnSearch'));
  btnSearch.addEventListener('click', @onSearch);
  edtSearch:=TJSHTMLInputElement(GetHTMLElement('edtSearch'));
  edtSearch.onkeyup:=@DoSearchKeyUp;
  edtSearch.addEventListener('focus',@DoSearchFocus);
  edtSearchInResults:=TJSHTMLInputElement(GetHTMLElement('edtSearchInResults'));
  edtSearchInResults.onkeyup:=@DoSearchResultsKeyUp;
  btnCloseSearchResult:=TJSHTMLButtonElement(GetHTMLElement('btnCloseSearchResult'));
  btnCloseSearchResult.addEventListener('click',@OnCloseSearch);
  FTextlayer:=GetHTMLElement('pdfTextlayer');
  divpdfViewer:=GetHTMLElement('pdfViewer');
end;


function TMyApplication.DoPageKeyUp(aEvent: TJSKeyBoardEvent): boolean;

Var
  aPage : Integer;

begin
  if not assigned(pdfDoc) then exit;
  Result:=False;
  if (aEvent.Key<>TJSKeyNames.Enter) then
    exit;
  aPage:=StrToIntDef(edtPageNo.Value,0);
  if (aPage>0) and (aPage<=PdfDoc.numPages) then
    queueRenderPage(aPage);
end;

procedure TMyApplication.onLoad(aEvent: TJSEvent);
begin
  StartPDFRender;
end;


procedure TMyApplication.DoSearch;

  Function OnHaveResults(aValue : JSValue) : JSValue;

  var
    Results : TPDFSearchResult absolute aValue;
    Pages : Array of Integer;
    I : Integer;

  begin
    Result:=Undefined;
    If Length(Results.Matches)>0 then
      begin
      Pages:=[Results.Matches[0].Page];
      // Create page number list
      For I:=1 to Length(Results.Matches)-1 do
        if TJSArray(Pages).indexOf(Results.Matches[i].Page)=-1 then
          TJSArray(Pages).Push(Results.Matches[i].Page);
      FSearchResultsPopover.classList.add('is-popover-active');
      FSearchResultPages:=Pages;
      ClearResultPane;
      For I:=1 to Length(Results.Matches)-1 do
        ShowResultMatch(I,Results.Matches[i]);
      ShowResultPane;
      ShowSearchResultMatchCount;
      ShowSearchResultPage(0);
      end;
  end;

var
  aterm : string;

begin
  if not assigned(pdfDoc) then exit;
  aTerm:=edtSearch.Value;
  if Length(aTerm)<=1 then exit;
  if FSearch=Nil then;
    FSearch:=TPDFSearch.Create(Self);
  FSearch.PDF:=pdfDoc;
  FSearch.searchDocument(aTerm)._then(@OnHaveResults);
end;



procedure TMyApplication.onSearch(aEvent: TJSEvent);
begin
  DoSearch;
end;

procedure TMyApplication.onClosePane(aEvent : TJSEvent);

begin
  HideResultPane;
end;

procedure TMyApplication.onCloseSearch(aEvent: TJSEvent);
begin
  FSearchResultsPopover.classList.Remove('is-popover-active');
end;

procedure TMyApplication.onNextSearchResult(aEvent: TJSEvent);
begin
  if FCurrentSearchResultPage>=(Length(FSearchResultPages)-1) then
    exit;
  ShowSearchResultPage(FCurrentSearchResultPage+1);
end;

procedure TMyApplication.onPreviousSearchResult(aEvent: TJSEvent);
begin
  if FCurrentSearchResultPage<1 then
    exit;
  ShowSearchResultPage(FCurrentSearchResultPage-1);
end;

procedure TMyApplication.ShowSearchResultMatchCount;

begin
  lblSearchResultCount.InnerText:=IntToStr(Length(FSearchResultPages));
end;


Procedure TMyApplication.DoShowMatch(aEvent : TJSEvent);

var
  aPage : Integer;

begin
  pnlResults.querySelectorAll('result-item').foreach(@MakeInactive);
  aEvent.currentTargetElement.classList.add('is-active');
  aPage:=StrToIntDef(String(TJSHTMLElement(aEvent.currentTargetElement).Dataset['page']),-1);
  if aPage<>-1 then
    queueRenderPage(aPage);
end;

Function TMyApplication.ShowResultMatch(Idx : Integer; aResult : TPDFSearchMatch) : TJSHTMLElement;

Const
  ResultContent =
  '<span class="panel-icon">'+
  '  <i class="las la-book" aria-hidden="true"></i>'+
  '</span>'+
  '<div class="subtitles">'+
  '  <p class="has-text-weight-semibold">Page %d:</p>'+
  '  <p>'+
  '    <span>%s</span>'+
  '    <span class="has-text-weight-bold">%s</span>'+
  '    <span>%s</span>'+
  '  </p>'+
  '</div>';

begin
  Result:=TJSHTMLElement(Document.CreateElement('a'));
  Result.Dataset['page']:=IntToStr(aResult.Page);
  Result.Dataset['idx']:=IntToStr(Idx);
  Result.ClassName:='panel-block result-item';
  Result.InnerHTML:=Format(ResultContent,[aResult.Page,aResult.Context1,aResult.Text,aResult.Context2]);
  Result.AddEventListener('click',@DoShowMatch);
  pnlResults.appendChild(Result);
end;

procedure TMyApplication.ShowResultPane;

begin
  pnlSidebar.classList.Remove('sidebar_closed');
  edtSearchInResults.Value:='';
end;

procedure TMyApplication.HideResultPane;
begin
  pnlSidebar.classList.Add('sidebar_closed');
end;


procedure TMyApplication.ClearResultPane;

Var
  Rem,Child : TJSHTMLElement;

begin
  Child:=TJSHTMLElement(pnlResults.firstElementChild);
  While Assigned(Child) do
    begin
    Rem:=TJSHTMLElement(Child);
    Child:=TJSHTMLElement(Rem.nextElementSibling);
    if Rem.classlist.contains('result-item')  then
      pnlResults.removeChild(Rem);
    end;
end;

procedure TMyApplication.ShowSearchResultPage(aIdx: Integer);

begin
  FCurrentSearchResultPage:=aIdx;
  lblCurrentSearchResult.InnerText:=IntToStr(aIdx+1);
  queueRenderPage(FSearchResultPages[aIdx]);
end;

function TMyApplication.DoSearchKeyUp(aEvent: TJSKeyBoardEvent): boolean;

begin
  Result:=False;
  if (aEvent.Key<>TJSKeyNames.Enter) then
    exit;
  DoSearch;
end;

procedure TMyApplication.DoSearchFocus(aEvent: TJSEvent);
begin
  FSearchResultsPopover.classList.Remove('is-popover-active');
end;

Function TMyApplication.DoSearchResultsKeyUp(aEvent: TJSKeyBoardEvent): boolean;

Var
  I,J,Len : Integer;
  List1,List2 : TJSNodeList;
  Pnl,Span : TJSHTMLElement;
  Vis : Boolean;
  aTerm : String;

begin
  Result:=True;
  aTerm:=UpperCase(edtSearchInResults.value);
  List1:=pnlResults.querySelectorAll('.result-item');
  for I:=0 to List1.Length-1 do
    begin
    Pnl:=TJSHTMLElement(List1.item(I));
    List2:=pnl.querySelectorAll('div p span');
    Vis:=(aTerm='');
    Len:=List2.Length;
    J:=0;
    While (Not Vis) and (J<Len) do
      begin
      Span:=TJSHTMLElement(List2.item(J));
      if (ATerm='') or (Pos(aTerm,UpperCase(Span.InnerText))>0) then
        Vis:=True;
      Inc(J);
      end;
    if vis then
      pnl.ClassList.Remove('is-hidden')
    else
      pnl.ClassList.add('is-hidden');
    end;
end;


var
  Application : TMyApplication;

begin
  Application:=TMyApplication.Create(nil);
  Application.Initialize;
  Application.Run;
end.
