unit searchpdf;

{$mode ObjFPC}

interface

uses
  Classes, SysUtils, pdfjs, js, types;

Type
  TPDFSearchMatch = Record
    Page : Integer;
    Text,
    Context1,
    Context2 : String;
  end;
  TPDFSearchMatchArray = Array of TPDFSearchMatch;

  TPDFSearchResult = Record
    Matches : TPDFSearchMatchArray;
  end;

  TPDFSearchResultArray = Array of TPDFSearchResult;
  TPDFSearchPageResultsEvent = Reference to Procedure(aPage : Integer; Matches : TPDFSearchMatchArray);

  { TPDFSearch }

  TPDFSearch = Class(TComponent)
  Private
    FOnPageResult: TPDFSearchPageResultsEvent;
    FPDF : TPDFDocumentProxy;
    FContextChars : Integer;
  Public
    function searchPage(aPageNo : Integer; aSearchTerm : String) : TPDFPromise;
    function searchDocument(aSearchTerm : String) : TJSPromise;
    Property PDF : TPDFDocumentProxy Read FPDF Write FPDF;
    Property ContextChars : Integer Read FContextChars Write FContextChars;
    Property OnPageResult : TPDFSearchPageResultsEvent Read FOnPageResult Write FOnPageResult;
  end;

implementation

function TPDFSearch.searchPage(aPageNo: Integer; aSearchTerm: String
  ): TPDFPromise;

  Function GotPage(aValue: JSValue) : JSValue;

  Var
    aPage : TPDFPageProxy absolute aValue;

  begin
    Result:=aPage.getTextContent();
  end;

  Function GotContent(aValue: JSValue) : JSValue;

    Function DoMap(Itm : JSValue; index: NativeInt; anArray : TJSArray) : JSValue;
    var
      aItem : TTextContentItem absolute itm;

    begin
      Result:=aItem.Str;
    end;

  Var
    aContent : TTextContent absolute aValue;
    aTerm,aReg, aText : String;
    aRegEx : TJSRegexp;
    aRegexMatches : TStringDynArray;
    Res : TPDFSearchResult;
    aMatch : TPDFSearchMatch;


  begin
    Res:=Default(TPDFSearchResult);
    aText:=TJSArray(TJSArray(aContent.items).map(@DoMap)).join('');
    aTerm:=aSearchTerm; // Todo: escape specials
    if FContextChars=0 then
      FContextChars:=30;
    aReg:=Format('(.{0,%d})%s(.{0,%d})',[FContextChars,aTerm,FContextChars]);
    aRegEx:=TJSRegExp.New(aReg,'gi');
    aRegexMatches:= aRegEx.exec(aText);
    While Length(aRegexMatches)>0 do
      begin
      aMatch:=Default(TPDFSearchMatch);
      aMatch.Page:=aPageNo;
      aMatch.Text:=aSearchTerm;
      if isString(aRegexMatches[1]) then
        aMatch.Context1:=aRegexMatches[1];
      if isString(aRegexMatches[2]) then
        aMatch.Context2:=aRegexMatches[2];
      Res.Matches:=Concat(Res.Matches,[aMatch]);
      aRegexMatches:=aRegEx.exec(aText);
      end;
    Result:=Res;
    If Assigned(FOnPageResult) then
      FOnPageResult(aPageNo,Res.Matches);
  end;

begin
  Result:= PDF.getPage(aPageNo).&then(@GotPage).&then(@GotContent);
end;

function TPDFSearch.searchDocument(aSearchTerm: String): TJSPromise;

  Function GotAllResults(aValue : JSValue) : JSValue;

  Var
    AllRes : TPDFSearchResultArray absolute aValue;
    aRes : TPDFSearchResult;
    DocumentResults : TPDFSearchResult;

  begin
    DocumentResults:=Default(TPDFSearchResult);
    for aRes in AllRes do
      DocumentResults.Matches:=Concat(DocumentResults.Matches,aRes.Matches);
    Result:=DocumentResults;
  end;

var
  results : array of TPDFPromise;
  I : Integer;

begin
  Results:=[];
  For I:=1 to PDF.numPages do
    Results:=Concat(Results,[SearchPage(I, aSearchterm)]);
  Result:=TJSPromise.all(TJSPromiseArray(Results))._then(@GotAllResults);
end;


end.

