program simplepdf;

{$mode objfpc}

uses
  browserapp, JS, Classes, SysUtils, Web, pdfjs;

Const
  ScaleStep = 0.2;

var
  pdfjsLib : TPDFJSStatic; external name 'globalThis.pdfjsLib';

type

  { TMyApplication }

  TMyApplication = class(TBrowserApplication)
    lblZoom : TJSHTMLElement;
    btnLoad,
    btnZoomIn,
    btnZoomOut,
    btnPrevious,
    btnFirst,
    btnNext,
    btnLast : TJSHTMLButtonElement;
    edtPageNo : TJSHTMLInputElement;
    pdfDoc : TPDFDocumentProxy;
    FPageNum : Integer; // 1
    FpageRendering : Boolean; // False
    FpageNumPending : Integer;// -1
    Fscale : Double ; // 0.8,
    FCanvas : TJSHTMLCanvasElement;
    FCtx : TJSCanvasRenderingContext2D; // canvas.getContext('2d');
    procedure doRun; override;
    procedure BindElements;
  private
    procedure DisplayZoom;
    function DoPageKeyUp(aEvent: TJSKeyBoardEvent): boolean;
    procedure onLoad(aEvent: TJSEvent);
    procedure onFirstPage(aEvent: TJSEvent);
    procedure onLastPage(aEvent: TJSEvent);
    procedure onNextPage(aEvent: TJSEvent);
    procedure onPrevPage(aEvent: TJSEvent);
    procedure onZoomIn(aEvent: TJSEvent);
    procedure onZoomOut(aEvent: TJSEvent);
    procedure queueRenderPage(aNum: Integer);
    procedure renderPage(aNum: Integer);
    procedure ShowPDF(aSource: TPDFSource);
    procedure StartPDFRender;
  end;

procedure TMyApplication.renderPage(aNum: Integer);

  function renderOK(aValue : JSValue) : JSValue;

  Var
    N : Integer;

  begin
    FPageRendering:=false;
    if (FPageNumPending <> -1) then
      begin
      N:=FPageNumPending;
      FpageNumPending:=-1;
      renderPage(N);
      end;
    Result:=True;
  end;

  function havePage (aValue : JSValue) : JSValue;

  var
    page : TPDFPageProxy absolute aValue;
    viewport : TPDFPageViewport;
    renderContext: TPDFRenderParams;
    renderTask : TPDFRenderTask;
    viewportParams : TViewportParameters;

  begin
    viewportParams:=TViewportParameters.new;
    viewportParams.scale:=FScale;
    viewport:=page.getViewport(viewportParams);
    Fcanvas.height := viewport.height;
    Fcanvas.width := viewport.width;
    renderContext:=TPDFRenderParams.New;
    renderContext.canvasContext:=Fctx;
    renderContext.viewport:=viewport;
    renderTask:=page.render(renderContext);
    renderTask.promise.&then(@renderOK);
    Result:=True;
  end;

begin
  FpageRendering:=True;
  pdfDoc.getPage(aNum).&then(@HavePage);
  edtPageNo.Value:=IntToStr(anum);
end;

procedure TMyApplication.queueRenderPage(aNum: Integer);

begin
  if FPageRendering then
    FpageNumPending:=aNum
  else
    renderPage(aNum)
end;

procedure TMyApplication.onPrevPage(aEvent : TJSEvent);

begin
  if (FpageNum <= 1) then exit;
  Dec(FpageNum);
  queueRenderPage(FpageNum);
end;


procedure TMyApplication.onFirstPage(aEvent : TJSEvent);

begin
  if not assigned(pdfDoc) then exit;
  FPageNum:=1;
  queueRenderPage(FpageNum);
end;

procedure TMyApplication.onLastPage(aEvent : TJSEvent);

begin
  if not assigned(pdfDoc) then exit;
  FPageNum:=pdfDoc.numPages;
  queueRenderPage(FpageNum);
end;

procedure TMyApplication.onNextPage(aEvent : TJSEvent);

begin
  if not assigned(pdfDoc) then exit;
  if (FpageNum >= pdfDoc.numPages) then exit;
  Inc(FpageNum);
  queueRenderPage(FpageNum);
end;

procedure TMyApplication.onZoomIn(aEvent : TJSEvent);

begin
  if not assigned(pdfDoc) then exit;
  FScale:=FScale+ScaleStep;
  DisplayZoom;
  queueRenderPage(FpageNum);
end;

procedure TMyApplication.onZoomOut(aEvent : TJSEvent);

begin
  if not assigned(pdfDoc) then exit;
  if FScale<=ScaleStep*2 then
    exit;
  FScale:=FScale-ScaleStep;
  DisplayZoom;
  queueRenderPage(FpageNum);
end;

procedure TMyApplication.DisplayZoom;
begin
  lblZoom.innerText:=IntToStr(Round(FScale*100))+'%';
end;

procedure MakeInactive(currentValue : TJSNode; currentIndex: NativeInt; list : TJSNodeList);

begin
  TJSHTMLElement(CurrentValue).classList.add('is-active');
end;

procedure TMyApplication.ShowPDF(aSource: TPDFSource);

  function pdfLoaded(aValue : JSValue) : JSValue;
  begin
    pdfDoc:=TPDFDocumentProxy(aValue);
    Writeln('PDF loaded');
    FpageNum:=1;
    QueueRenderPage(FpageNum);
    Result:=True;
  end;

  Procedure pdfLoadError(aReason : String);

  begin
    Writeln('PDF loading failed: '+aReason);
  end;

var
  loadingTask : TPDFLoadingTask;

begin
  loadingTask:=pdfjsLib.getDocument(aSource);
  loadingTask.promise.&then(@pdfLoaded,@pdfLoadError);
end;

procedure TMyApplication.StartPDFRender;

var
  aSource : TPDFSource;
  aURL : String;

begin
{  aUrl:=window.prompt('Please enter the URL of a PDF to view');
  if IsNull(aURL) then
    exit;}
  aUrl:='sample.pdf';
  aSource:=TPDFSource.New;
  aSource.URL:=aURL;
  ShowPdf(aSource);
end;

procedure TMyApplication.doRun;

begin
  FPageNum:=1;
  FPageRendering:=False;
  FPageNumPending:= -1;
  FScale:=1;
  pdfjsLib.GlobalWorkerOptions.workerSrc:= 'https://mozilla.github.io/pdf.js/build/pdf.worker.js';
  pdfjsLib.GlobalWorkerOptions.workerSrc:= 'https://cdnjs.cloudflare.com/ajax/libs/pdf.js/4.0.269/pdf.worker.mjs';
  BindElements;
  StartPDFRender;
  DisplayZoom;
  Terminate;

end;

procedure TMyApplication.BindElements;
begin
  btnPrevious:=TJSHTMLButtonElement(GetHTMLElement('btnPrevious'));
  btnPrevious.addEventListener('click', @onPrevPage);
  btnFirst:=TJSHTMLButtonElement(GetHTMLElement('btnFirst'));
  btnFirst.addEventListener('click', @onFirstPage);
  btnNext:=TJSHTMLButtonElement(GetHTMLElement('btnNext'));
  btnNext.addEventListener('click', @onNextPage);
  btnLast:=TJSHTMLButtonElement(GetHTMLElement('btnLast'));
  btnLast.addEventListener('click', @onLastPage);
  btnLoad:=TJSHTMLButtonElement(GetHTMLElement('btnLoad'));
  btnLoad.addEventListener('click', @onLoad);

  btnZoomIn:=TJSHTMLButtonElement(GetHTMLElement('btnZoomIn'));
  btnZoomIn.addEventListener('click', @onZoomIn);
  btnZoomOut:=TJSHTMLButtonElement(GetHTMLElement('btnZoomOut'));
  btnZoomOut.addEventListener('click', @onZoomOut);
  lblZoom:=GetHTMLElement('lblZoom');

  edtPageNo:=TJSHTMLInputElement(GetHTMLElement('edtPageNo'));
  edtPageNo.onkeyup:=@DoPageKeyUp;
  FCanvas:=TJSHTMLCanvasElement(GetHTMLElement('PDFCanvas'));
  FCtx:=TJSCanvasRenderingContext2D(FCanvas.getContext('2d'));
end;

function TMyApplication.DoPageKeyUp(aEvent: TJSKeyBoardEvent): boolean;

Var
  aPage : Integer;

begin
  if not assigned(pdfDoc) then exit;
  Result:=False;
  if (aEvent.Key<>TJSKeyNames.Enter) then
    exit;
  aPage:=StrToIntDef(edtPageNo.Value,0);
  if (aPage>0) and (aPage<=PdfDoc.numPages) then
    queueRenderPage(aPage);
end;

procedure TMyApplication.onLoad(aEvent: TJSEvent);
begin
  StartPDFRender;
end;

var
  Application : TMyApplication;

begin
  Application:=TMyApplication.Create(nil);
  Application.Initialize;
  Application.Run;
end.
